CREATE TABLE Staff (
	staff_id SERIAL PRIMARY KEY,
	given_name TEXT,
	family_name TEXT,
	role TEXT
);
CREATE TABLE Branch (
	name TEXT PRIMARY KEY,
	location TEXT,
	manager INT REFERENCES Staff ON DELETE SET NULL ON UPDATE CASCADE
);
CREATE TABLE WorksAt (
	branch_id TEXT REFERENCES Branch ON UPDATE CASCADE ON DELETE CASCADE,
	staff_id INT REFERENCES Staff ON UPDATE CASCADE ON DELETE CASCADE,
	PRIMARY KEY 	(branch_id, staff_id)
);
CREATE TABLE Customer (
	customer_id SERIAL PRIMARY KEY,
	branch_id TEXT REFERENCES Branch ON UPDATE CASCADE ON DELETE CASCADE,
	given_name TEXT,
	family_name TEXT,
	gender TEXT,
	dob DATE,
	age INT,
	status TEXT
);
CREATE TABLE Account (
	account_id BIGSERIAL PRIMARY KEY,
	customer_id INT REFERENCES Customer ON UPDATE CASCADE ON DELETE RESTRICT,
	balance NUMERIC,
	opened_by INT REFERENCES Staff ON UPDATE CASCADE ON DELETE RESTRICT
);
CREATE TABLE Transactions (
	transaction_id BIGSERIAL PRIMARY KEY,
	account_id BIGINT REFERENCES Account ON UPDATE CASCADE ON DELETE CASCADE,
	transaction NUMERIC
);

INSERT INTO Staff VALUES (DEFAULT, 'Isaac', 'Asmiov', 'Manager');
INSERT INTO Staff VALUES (DEFAULT, 'Ray', 'Bradbury', 'Teller');
INSERT INTO Staff VALUES (DEFAULT, 'Arthur', 'Clarke', 'Teller');
INSERT INTO Staff VALUES (DEFAULT, 'Samuel', 'Delany', 'Customer Service');
INSERT INTO Staff VALUES (DEFAULT, 'Greg', 'Egan', 'Teller');

INSERT INTO Branch VALUES ('Canberra', '1');
INSERT INTO Branch VALUES ('Brisbane', '2');
INSERT INTO Branch VALUES ('Sydney');
INSERT INTO Branch VALUES ('Melbourne');

INSERT INTO WorksAt VALUES ('Canberra', '1');
INSERT INTO WorksAt VALUES ('Brisbane', '2');
INSERT INTO WorksAt VALUES ('Sydney', '3');
INSERT INTO WorksAt VALUES ('Melbourne', '2');

INSERT INTO Customer VALUES (DEFAULT, 'Canberra', 'Esther', 'Friesner', 'Female', '01-01-1980', NULL, 'Current');
INSERT INTO Customer VALUES (DEFAULT, 'Brisbane', 'William', 'Gibson', 'Male', '01-01-1970', 37, 'Current');
INSERT INTO Customer VALUES (DEFAULT, 'Sydney', 'Peter', 'Hamilton', 'Male', '01-01-1969', 38, 'Current');
INSERT INTO Customer VALUES (DEFAULT, NULL, 'Simon', 'Ings', 'Male', '01-01-1971', 36, 'Current');
INSERT INTO Customer VALUES (DEFAULT, NULL, 'Gweneth', 'Jones', 'Female', '01-01-1980', NULL, 'Current');

INSERT INTO Account VALUES (DEFAULT, '1', NULL, '1');
INSERT INTO Account VALUES (DEFAULT, '1', NULL, '1');
INSERT INTO Account VALUES (DEFAULT, '2', NULL, '1');
INSERT INTO Account VALUES (DEFAULT, '3', NULL, '2');
INSERT INTO Account VALUES (DEFAULT, '4', NULL, '2');
INSERT INTO Account VALUES (DEFAULT, '5', NULL, '3');

INSERT INTO Transactions VALUES (DEFAULT, '1', '100.00');
INSERT INTO Transactions VALUES (DEFAULT, '1', '200.00');
INSERT INTO Transactions VALUES (DEFAULT, '1', '-50.00');
INSERT INTO Transactions VALUES (DEFAULT, '2', '100.00');
INSERT INTO Transactions VALUES (DEFAULT, '2', '200.00');
INSERT INTO Transactions VALUES (DEFAULT, '2', '200.00');
INSERT INTO Transactions VALUES (DEFAULT, '2', '-500.00');
INSERT INTO Transactions VALUES (DEFAULT, '2', '100.00');
INSERT INTO Transactions VALUES (DEFAULT, '3', '100.00');
INSERT INTO Transactions VALUES (DEFAULT, '3', '200.00');
INSERT INTO Transactions VALUES (DEFAULT, '3', '200.00');
INSERT INTO Transactions VALUES (DEFAULT, '3', '150.00');
INSERT INTO Transactions VALUES (DEFAULT, '3', '-50.00');
INSERT INTO Transactions VALUES (DEFAULT, '3', '-50.00');
INSERT INTO Transactions VALUES (DEFAULT, '4', '100.00');
INSERT INTO Transactions VALUES (DEFAULT, '4', '200.00');
INSERT INTO Transactions VALUES (DEFAULT, '4', '200.00');
INSERT INTO Transactions VALUES (DEFAULT, '4', '-250.00');
INSERT INTO Transactions VALUES (DEFAULT, '4', '-50.00');
INSERT INTO Transactions VALUES (DEFAULT, '5', '100.00');
INSERT INTO Transactions VALUES (DEFAULT, '5', '200.00');
INSERT INTO Transactions VALUES (DEFAULT, '5', '200.00');
INSERT INTO Transactions VALUES (DEFAULT, '5', '250.00');
INSERT INTO Transactions VALUES (DEFAULT, '5', '250.00');
INSERT INTO Transactions VALUES (DEFAULT, '5', '200.00');
INSERT INTO Transactions VALUES (DEFAULT, '1', '-2000.00');

GRANT ALL ON Staff TO osdc;
GRANT ALL ON Branch TO osdc;
GRANT ALL ON WorksAt TO osdc;
GRANT ALL ON Customer TO osdc;
GRANT ALL ON Account TO osdc;
GRANT ALL ON Transactions TO osdc;
GRANT ALL ON account_account_id_seq TO osdc;
GRANT ALL ON customer_customer_id_seq TO osdc;
GRANT ALL ON staff_staff_id_seq TO osdc;
GRANT ALL ON transactions_transaction_id_seq TO osdc;
